searchData={"items":[{"type":"module","title":"Buckets","doc":"Provides a generic interface for uploading files to buckets hosted by\ndifferent cloud providers.","ref":"Buckets.html"},{"type":"function","title":"Buckets.delete/2","doc":"Delegates a `delete/2` function call to the configured `:adapter`.","ref":"Buckets.html#delete/2"},{"type":"function","title":"Buckets.get/2","doc":"Delegates a `get/2` function call to the configured `:adapter`.","ref":"Buckets.html#get/2"},{"type":"function","title":"Buckets.put/3","doc":"Delegates a `put/3` function call to the configured `:adapter`.","ref":"Buckets.html#put/3"},{"type":"function","title":"Buckets.url/2","doc":"Delegates a `url/2` function call to the configured `:adapter`.","ref":"Buckets.html#url/2"},{"type":"behaviour","title":"Buckets.Adapter","doc":"Behaviour definition for Buckets storage adapters.\n\nAn adapter implements the storage operations for a specific backend (filesystem,\nS3, GCS, etc.). All adapters must implement the required callbacks to handle\nfile operations like uploading, downloading, and generating signed URLs.","ref":"Buckets.Adapter.html"},{"type":"behaviour","title":"Implementing an Adapter - Buckets.Adapter","doc":"To create a new adapter, implement this behaviour:\n\n    defmodule MyApp.CustomAdapter do\n      @behaviour Buckets.Adapter\n\n      @impl true\n      def validate_config(config) do\n        # Validate and normalize configuration\n      end\n\n      @impl true\n      def put(object, remote_path, config) do\n        # Upload object to storage\n      end\n\n      @impl true\n      def get(remote_path, config) do\n        # Download data from storage\n      end\n\n      @impl true\n      def url(remote_path, config) do\n        # Generate signed URL for direct access\n      end\n\n      @impl true\n      def delete(remote_path, config) do\n        # Delete object from storage\n      end\n\n      # Optional: Only if adapter needs supervised processes\n      @impl true\n      def child_spec(config, cloud_module) do\n        # Return supervisor child specification\n      end\n    end","ref":"Buckets.Adapter.html#module-implementing-an-adapter"},{"type":"behaviour","title":"Configuration - Buckets.Adapter","doc":"Each adapter defines its own configuration options, but common ones include:\n- `:bucket` - The storage bucket/container name\n- `:path` - Base path within the bucket\n- `:uploader` - Uploader type for LiveView direct uploads","ref":"Buckets.Adapter.html#module-configuration"},{"type":"behaviour","title":"Supervised Processes - Buckets.Adapter","doc":"Some adapters (like GCS) require background processes for authentication or\nconnection management. These adapters should implement the optional `child_spec/2`\ncallback. Adapters that don't need supervision (like Volume, S3) should not\nimplement this callback.","ref":"Buckets.Adapter.html#module-supervised-processes"},{"type":"callback","title":"Buckets.Adapter.child_spec/2","doc":"Returns a child specification for processes this adapter needs, if any.\n\nThis callback is optional. Adapters that need supervised processes (like GCS\nauth servers) should implement this callback. Adapters that don't need any\nsupervised processes (like Volume, S3) can omit this callback entirely.\n\nWhen implemented, should return:\n- `child_spec()` - A supervisor child specification\n- `{:error, reason}` - Config is invalid\n\nExamples:\n- GCS adapter implements this to return auth server child spec\n- Volume/S3 adapters don't implement this callback at all","ref":"Buckets.Adapter.html#c:child_spec/2"},{"type":"callback","title":"Buckets.Adapter.delete/2","doc":"Deletes an object from the storage backend.\n\nTakes the remote path of the object to delete and the adapter configuration.\nReturns `{:ok, metadata}` on success or `{:error, reason}` on failure.","ref":"Buckets.Adapter.html#c:delete/2"},{"type":"callback","title":"Examples - Buckets.Adapter.delete/2","doc":"def delete(\"uploads/file.pdf\", config) do\n      case delete_object(path, config) do\n        :ok -> {:ok, %{}}\n        {:error, reason} -> {:error, reason}\n      end\n    end","ref":"Buckets.Adapter.html#c:delete/2-examples"},{"type":"callback","title":"Buckets.Adapter.get/2","doc":"Downloads data from the storage backend.\n\nTakes the remote path of the object and the adapter configuration.\nReturns the binary data on success or an error tuple on failure.","ref":"Buckets.Adapter.html#c:get/2"},{"type":"callback","title":"Examples - Buckets.Adapter.get/2","doc":"def get(\"uploads/file.pdf\", config) do\n      case download_file(path, config) do\n        {:ok, data} -> {:ok, data}\n        {:error, :not_found} -> {:error, :not_found}\n        {:error, reason} -> {:error, reason}\n      end\n    end","ref":"Buckets.Adapter.html#c:get/2-examples"},{"type":"callback","title":"Buckets.Adapter.put/3","doc":"Uploads an object to the storage backend.\n\nTakes a `Buckets.Object` struct containing the data to upload, the remote\npath where it should be stored, and the adapter configuration.\n\nShould return `{:ok, metadata}` on success where metadata is adapter-specific,\nor `{:error, reason}` on failure.","ref":"Buckets.Adapter.html#c:put/3"},{"type":"callback","title":"Examples - Buckets.Adapter.put/3","doc":"def put(object, \"uploads/file.pdf\", config) do\n      # Upload logic here\n      {:ok, %{etag: \"abc123\", version: \"v1\"}}\n    end","ref":"Buckets.Adapter.html#c:put/3-examples"},{"type":"callback","title":"Buckets.Adapter.url/2","doc":"Generates a signed URL for direct access to an object.\n\nTakes the remote path and configuration (which may include options like\n`:expires_in` for controlling URL expiration).\n\nReturns a `Buckets.SignedURL` struct containing the URL and metadata.","ref":"Buckets.Adapter.html#c:url/2"},{"type":"callback","title":"Options - Buckets.Adapter.url/2","doc":"- `:expires_in` - URL expiration time in seconds (adapter-specific default)\n- `:for_upload` - Generate a URL for uploading instead of downloading","ref":"Buckets.Adapter.html#c:url/2-options"},{"type":"callback","title":"Examples - Buckets.Adapter.url/2","doc":"def url(\"uploads/file.pdf\", config) do\n      signed_url = generate_signed_url(path, config)\n      {:ok, %Buckets.SignedURL{url: signed_url, expires_at: expiry}}\n    end","ref":"Buckets.Adapter.html#c:url/2-examples"},{"type":"callback","title":"Buckets.Adapter.validate_config/1","doc":"Validates and normalizes adapter configuration.\n\nThis callback is called during Cloud module initialization to ensure the\nconfiguration is valid. It should:\n- Validate required options are present\n- Set default values for optional options\n- Normalize configuration format\n- Return an error tuple with invalid/missing keys if validation fails","ref":"Buckets.Adapter.html#c:validate_config/1"},{"type":"callback","title":"Examples - Buckets.Adapter.validate_config/1","doc":"def validate_config(config) do\n      config\n      |> Keyword.validate([:adapter, :bucket, :path, :uploader])\n      |> case do\n        {:ok, config} -> validate_required(config, [:bucket])\n        error -> error\n      end\n    end","ref":"Buckets.Adapter.html#c:validate_config/1-examples"},{"type":"function","title":"Buckets.Adapter.validate_required/2","doc":"Validates that required configuration keys are present.\n\nA helper function for adapters to use in their `validate_config/1` callback\nto ensure all required configuration options are provided.","ref":"Buckets.Adapter.html#validate_required/2"},{"type":"function","title":"Parameters - Buckets.Adapter.validate_required/2","doc":"- `config` - The configuration keyword list to validate\n- `required` - List of required keys that must be present in the config","ref":"Buckets.Adapter.html#validate_required/2-parameters"},{"type":"function","title":"Returns - Buckets.Adapter.validate_required/2","doc":"- `{:ok, config}` - All required keys are present\n- `{:error, missing_keys}` - List of keys that are missing from config","ref":"Buckets.Adapter.html#validate_required/2-returns"},{"type":"function","title":"Examples - Buckets.Adapter.validate_required/2","doc":"def validate_config(config) do\n      with {:ok, config} <- Keyword.validate(config, @allowed_options),\n           {:ok, config} <- validate_required(config, [:bucket, :access_key]) do\n        {:ok, config}\n      end\n    end","ref":"Buckets.Adapter.html#validate_required/2-examples"},{"type":"function","title":"Implementation Note - Buckets.Adapter.validate_required/2","doc":"The function filters out `nil` values from the required list, allowing\nconditional validation based on other configuration options.","ref":"Buckets.Adapter.html#validate_required/2-implementation-note"},{"type":"module","title":"Buckets.Adapters.GCS","doc":"Google Cloud Storage adapter for Buckets.\n\nThis adapter provides a native implementation for GCS operations using\nonly `:req` as a dependency.","ref":"Buckets.Adapters.GCS.html"},{"type":"module","title":"Setup - Buckets.Adapters.GCS","doc":"To use this adapter, you need to start your Cloud module in your application's\nsupervision tree to enable automatic token caching and refresh:\n\n    children = [\n      # ... your other processes\n      MyApp.Cloud\n    ]\n\n    Supervisor.start_link(children, opts)\n\nThe Cloud module will automatically start the required authentication processes for GCS locations.\n\nThe supervisor will automatically manage authentication tokens for each unique set of\nservice account credentials, refreshing them before they expire.","ref":"Buckets.Adapters.GCS.html#module-setup"},{"type":"module","title":"Configuration - Buckets.Adapters.GCS","doc":"You can configure GCS using either service account credentials as a JSON string\nor by providing a path to a service account JSON file:\n\n    # Using credentials from environment variable (JSON string)\n    config :my_app, MyApp.Cloud,\n      adapter: Buckets.Adapters.GCS,\n      bucket: \"my-bucket\",\n      service_account_credentials: System.fetch_env!(\"GOOGLE_CREDENTIALS\")\n\n    # Using credentials from file path\n    config :my_app, MyApp.Cloud,\n      adapter: Buckets.Adapters.GCS,\n      bucket: \"my-bucket\",\n      service_account_path: \"/path/to/service-account.json\"","ref":"Buckets.Adapters.GCS.html#module-configuration"},{"type":"module","title":"Optional Configuration - Buckets.Adapters.GCS","doc":"config :my_app, MyApp.Cloud,\n      adapter: Buckets.Adapters.GCS,\n      bucket: \"my-bucket\",\n      service_account_credentials: System.fetch_env!(\"GOOGLE_CREDENTIALS\"),\n      # Optional: base path within bucket\n      path: \"uploads\",\n      # Optional: uploader module for LiveView direct uploads\n      uploader: \"GCS\"\n\nThe `:service_account_credentials` option accepts a JSON string, making it easy to pass\ncredentials via environment variables in production environments.","ref":"Buckets.Adapters.GCS.html#module-optional-configuration"},{"type":"module","title":"Performance - Buckets.Adapters.GCS","doc":"This implementation automatically caches and refreshes Google Cloud access tokens,\nsignificantly reducing latency compared to generating new tokens for each request.\nTokens are refreshed 5 minutes before expiration to ensure uninterrupted service.","ref":"Buckets.Adapters.GCS.html#module-performance"},{"type":"function","title":"Buckets.Adapters.GCS.url/2","doc":"Gets a signed URL for temporarily delegating access to an object in a bucket.","ref":"Buckets.Adapters.GCS.html#url/2"},{"type":"function","title":"Errors - Buckets.Adapters.GCS.url/2","doc":"* 403 PERMISSION_DENIED: Permission 'iam.serviceAccounts.signBlob' denied on resource\n      (or it may not exist). Make sure the authorized SA has role roles/iam.serviceAccountTokenCreator\n      on the SA passed in the URL.\n\n    * 403 PERMISSION_DENIED: IAM Service Account Credentials API has not been used in project {project_id}\n      before or it is disabled. Enable it by visiting {url} then retry. If you enabled this API recently,\n      wait a few minutes for the action to propagate to our systems and retry. Make sure the authorized SA\n      has role roles/iam.serviceAccountTokenCreator on the SA passed in the URL.\"","ref":"Buckets.Adapters.GCS.html#url/2-errors"},{"type":"module","title":"Buckets.Adapters.GCS.Auth","doc":"Google Cloud Storage authentication using service account credentials.\n\nHandles JWT token generation and OAuth2 token exchange without external dependencies.","ref":"Buckets.Adapters.GCS.Auth.html"},{"type":"function","title":"Buckets.Adapters.GCS.Auth.exchange_jwt_for_token/1","doc":"Exchanges a JWT token for an access token via Google's OAuth2 endpoint.","ref":"Buckets.Adapters.GCS.Auth.html#exchange_jwt_for_token/1"},{"type":"function","title":"Buckets.Adapters.GCS.Auth.generate_jwt/1","doc":"Generates a JWT token for Google service account authentication.","ref":"Buckets.Adapters.GCS.Auth.html#generate_jwt/1"},{"type":"function","title":"Buckets.Adapters.GCS.Auth.get_access_token/1","doc":"Gets an access token for GCS API access.","ref":"Buckets.Adapters.GCS.Auth.html#get_access_token/1"},{"type":"function","title":"Examples - Buckets.Adapters.GCS.Auth.get_access_token/1","doc":"iex> credentials = %{\n    ...>   \"client_email\" => \"test@example.com\",\n    ...>   \"private_key\" => \"-----BEGIN PRIVATE KEY-----\\n...\\n-----END PRIVATE KEY-----\"\n    ...> }\n    iex> Buckets.GCS.Auth.get_access_token(credentials)\n    {:ok, \"ya29.c.Ko8...\"}","ref":"Buckets.Adapters.GCS.Auth.html#get_access_token/1-examples"},{"type":"function","title":"Buckets.Adapters.GCS.Auth.get_credentials/1","doc":"Get service account credentials from a location config.","ref":"Buckets.Adapters.GCS.Auth.html#get_credentials/1"},{"type":"function","title":"Buckets.Adapters.GCS.Auth.load_credentials/1","doc":"Loads service account credentials from a file path.","ref":"Buckets.Adapters.GCS.Auth.html#load_credentials/1"},{"type":"function","title":"Buckets.Adapters.GCS.Auth.parse_credentials/1","doc":"Parse service account credentials from a JSON string.","ref":"Buckets.Adapters.GCS.Auth.html#parse_credentials/1"},{"type":"function","title":"Buckets.Adapters.GCS.Auth.validate_credentials/1","doc":"Validates that service account credentials contain required fields.","ref":"Buckets.Adapters.GCS.Auth.html#validate_credentials/1"},{"type":"module","title":"Buckets.Adapters.GCS.AuthServer","doc":"GenServer that manages Google Cloud Storage access tokens.\n\nAutomatically refreshes tokens before they expire to ensure uninterrupted access to GCS APIs.","ref":"Buckets.Adapters.GCS.AuthServer.html"},{"type":"function","title":"Buckets.Adapters.GCS.AuthServer.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Buckets.Adapters.GCS.AuthServer.html#child_spec/1"},{"type":"function","title":"Buckets.Adapters.GCS.AuthServer.get_token/1","doc":"Gets a valid access token from a specific server, refreshing if necessary.","ref":"Buckets.Adapters.GCS.AuthServer.html#get_token/1"},{"type":"function","title":"Buckets.Adapters.GCS.AuthServer.refresh_token/1","doc":"Forces a token refresh.","ref":"Buckets.Adapters.GCS.AuthServer.html#refresh_token/1"},{"type":"function","title":"Buckets.Adapters.GCS.AuthServer.start_link/1","doc":"Starts the token server.\n\nIf opts[:cloud] exists, starts in named mode with the cloud module.\nOtherwise, treats opts as config and starts in nameless mode.","ref":"Buckets.Adapters.GCS.AuthServer.html#start_link/1"},{"type":"module","title":"Buckets.Adapters.GCS.Signature","doc":"Google Cloud Storage V4 signed URL generation.\n\nImplements the V4 signing process for creating signed URLs that allow\ntemporary access to GCS objects without requiring authentication.","ref":"Buckets.Adapters.GCS.Signature.html"},{"type":"function","title":"Buckets.Adapters.GCS.Signature.generate_v4/4","doc":"Generates a V4 signed URL for Google Cloud Storage.","ref":"Buckets.Adapters.GCS.Signature.html#generate_v4/4"},{"type":"function","title":"Options - Buckets.Adapters.GCS.Signature.generate_v4/4","doc":"* `:verb` - HTTP method (default: \"GET\")\n  * `:expires` - Expiration time in seconds from now (default: 3600)\n  * `:headers` - Additional headers to include in the signature","ref":"Buckets.Adapters.GCS.Signature.html#generate_v4/4-options"},{"type":"function","title":"Examples - Buckets.Adapters.GCS.Signature.generate_v4/4","doc":"iex> credentials = %{\"client_email\" => \"test@example.com\", \"private_key\" => \"...\"}\n    iex> Buckets.Adapters.GCS.Signature.generate_v4(credentials, \"my-bucket\", \"path/to/object.jpg\")\n    {:ok, \"https://storage.googleapis.com/my-bucket/path%2Fto%2Fobject.jpg?X-Goog-Algorithm=...\"}","ref":"Buckets.Adapters.GCS.Signature.html#generate_v4/4-examples"},{"type":"module","title":"Buckets.Adapters.S3","doc":"Adapter for Amazon S3 and S3-compatible storage services.\n\nThis adapter supports multiple S3-compatible providers through the `:provider` option:\n- `:aws` (default) - Amazon S3\n- `:cloudflare` - Cloudflare R2\n- `:digitalocean` - DigitalOcean Spaces\n- `:tigris` - Tigris","ref":"Buckets.Adapters.S3.html"},{"type":"module","title":"Configuration - Buckets.Adapters.S3","doc":"","ref":"Buckets.Adapters.S3.html#module-configuration"},{"type":"module","title":"Amazon S3 (default) - Buckets.Adapters.S3","doc":"config :my_app, MyApp.Cloud,\n      adapter: Buckets.Adapters.S3,\n      bucket: \"my-bucket\",\n      region: \"us-east-1\",\n      access_key_id: System.fetch_env!(\"AWS_ACCESS_KEY_ID\"),\n      secret_access_key: System.fetch_env!(\"AWS_SECRET_ACCESS_KEY\")","ref":"Buckets.Adapters.S3.html#module-amazon-s3-default"},{"type":"module","title":"Cloudflare R2 - Buckets.Adapters.S3","doc":"config :my_app, MyApp.Cloud,\n      adapter: Buckets.Adapters.S3,\n      provider: :cloudflare,\n      bucket: \"my-bucket\",\n      endpoint_url: \"https://YOUR_ACCOUNT_ID.r2.cloudflarestorage.com\",\n      access_key_id: System.fetch_env!(\"R2_ACCESS_KEY_ID\"),\n      secret_access_key: System.fetch_env!(\"R2_SECRET_ACCESS_KEY\")\n\nNote: Cloudflare R2 automatically uses `region: \"auto\"`.","ref":"Buckets.Adapters.S3.html#module-cloudflare-r2"},{"type":"module","title":"DigitalOcean Spaces - Buckets.Adapters.S3","doc":"config :my_app, MyApp.Cloud,\n      adapter: Buckets.Adapters.S3,\n      provider: :digitalocean,\n      bucket: \"my-bucket\",\n      access_key_id: System.fetch_env!(\"SPACES_ACCESS_KEY\"),\n      secret_access_key: System.fetch_env!(\"SPACES_SECRET_KEY\")\n\nNote: DigitalOcean Spaces defaults to NYC3 region and endpoint. Override with:\n\n    config :my_app, MyApp.Cloud,\n      adapter: Buckets.Adapters.S3,\n      provider: :digitalocean,\n      bucket: \"my-bucket\",\n      region: \"sfo3\",\n      endpoint_url: \"https://sfo3.digitaloceanspaces.com\",\n      access_key_id: System.fetch_env!(\"SPACES_ACCESS_KEY\"),\n      secret_access_key: System.fetch_env!(\"SPACES_SECRET_KEY\")","ref":"Buckets.Adapters.S3.html#module-digitalocean-spaces"},{"type":"module","title":"Tigris - Buckets.Adapters.S3","doc":"config :my_app, MyApp.Cloud,\n      adapter: Buckets.Adapters.S3,\n      provider: :tigris,\n      bucket: \"my-bucket\",\n      region: \"auto\",\n      access_key_id: System.fetch_env!(\"TIGRIS_ACCESS_KEY_ID\"),\n      secret_access_key: System.fetch_env!(\"TIGRIS_SECRET_ACCESS_KEY\")\n\nNote: Tigris automatically uses the endpoint `https://fly.storage.tigris.dev`.","ref":"Buckets.Adapters.S3.html#module-tigris"},{"type":"module","title":"Required Options - Buckets.Adapters.S3","doc":"- `:bucket` - The S3 bucket name\n- `:access_key_id` - AWS access key ID or equivalent\n- `:secret_access_key` - AWS secret access key or equivalent\n- `:region` - AWS region (defaults to \"auto\" for some providers)\n- `:endpoint_url` - Required for non-AWS providers","ref":"Buckets.Adapters.S3.html#module-required-options"},{"type":"module","title":"Optional Options - Buckets.Adapters.S3","doc":"- `:provider` - The S3-compatible provider (defaults to `:aws`)\n- `:path` - Base path within the bucket\n- `:uploader` - Uploader configuration for direct uploads","ref":"Buckets.Adapters.S3.html#module-optional-options"},{"type":"module","title":"Supervision - Buckets.Adapters.S3","doc":"This adapter does not require any supervised processes. You do not need to add Cloud modules\nusing this adapter to your supervision tree.","ref":"Buckets.Adapters.S3.html#module-supervision"},{"type":"module","title":"Buckets.Adapters.Volume","doc":"Adapter for local filesystem storage.\n\nThe Volume adapter stores files on the local filesystem and is primarily intended\nfor development environments. It supports signed URLs and integrates with the\nBuckets dev router for handling uploads and downloads.","ref":"Buckets.Adapters.Volume.html"},{"type":"module","title":"Configuration - Buckets.Adapters.Volume","doc":"config :my_app, MyApp.Cloud,\n      adapter: Buckets.Adapters.Volume,\n      bucket: \"tmp/buckets_volume\",\n      base_url: \"http://localhost:4000\"","ref":"Buckets.Adapters.Volume.html#module-configuration"},{"type":"module","title":"Required Options - Buckets.Adapters.Volume","doc":"- `:bucket` - The local directory path where files will be stored\n- `:base_url` - The base URL for generating file URLs (e.g., your Phoenix app URL)","ref":"Buckets.Adapters.Volume.html#module-required-options"},{"type":"module","title":"Optional Options - Buckets.Adapters.Volume","doc":"- `:path` - Base path within the bucket directory\n- `:endpoint` - Custom endpoint path for signed URLs (defaults to \"/__buckets__\")\n- `:uploader` - Uploader configuration for direct uploads","ref":"Buckets.Adapters.Volume.html#module-optional-options"},{"type":"module","title":"Development Router Integration - Buckets.Adapters.Volume","doc":"To handle local uploads and downloads in development, add the Volume routes to your router:\n\n    # In your router.ex\n    if Application.compile_env(:my_app, :dev_routes) do\n      import Buckets.Router\n\n      buckets_volume(MyApp.Cloud)\n    end\n\nThis will mount routes at `/__buckets__/:bucket/*path` for handling:\n- File uploads (PUT requests)\n- File downloads (GET requests)","ref":"Buckets.Adapters.Volume.html#module-development-router-integration"},{"type":"module","title":"Signed URLs - Buckets.Adapters.Volume","doc":"The Volume adapter supports signed URLs for secure file uploads and downloads:\n\n    {:ok, object} = MyApp.Cloud.url(object, expires_in: 3600)\n    # Returns object with signed URL valid for 1 hour","ref":"Buckets.Adapters.Volume.html#module-signed-urls"},{"type":"module","title":"Supervision - Buckets.Adapters.Volume","doc":"This adapter does not require any supervised processes. Do not add Cloud modules\nusing this adapter to your supervision tree.","ref":"Buckets.Adapters.Volume.html#module-supervision"},{"type":"function","title":"Buckets.Adapters.Volume.verify_signed_path/3","doc":"","ref":"Buckets.Adapters.Volume.html#verify_signed_path/3"},{"type":"behaviour","title":"Buckets.Cloud","doc":"Defines a cloud.\n\nA cloud manages the movement of files and data between your application and\na remote bucket for persistent storage.\n\nWhen used, it expects `:otp_app` as an option:\n\n    defmodule MyApp.Cloud do\n      use Buckets.Cloud,\n        otp_app: :my_app\n    end\n\nConfiguration is fetched from the application config, using a combination of\n`:otp_app` and the module that you defined:\n\n    config :my_app, MyApp.Cloud,\n      adapter: Buckets.Adapters.Volume,\n      bucket: \"tmp/buckets_volume\",\n      base_url: \"http://localhost:4000\"\n\nEach Cloud module corresponds to a single storage backend, similar to how\nEcto.Repo modules correspond to a single database. For multi-cloud applications,\ndefine multiple Cloud modules:\n\n    defmodule MyApp.VolumeCloud do\n      use Buckets.Cloud, otp_app: :my_app\n    end\n\n    defmodule MyApp.GCSCloud do\n      use Buckets.Cloud, otp_app: :my_app\n    end\n\n    defmodule MyApp.S3Cloud do\n      use Buckets.Cloud, otp_app: :my_app\n    end\n\nYou may also specify config dynamically at runtime, using the `:config` opt\nwhere it is supported.","ref":"Buckets.Cloud.html"},{"type":"behaviour","title":"Supervision - Buckets.Cloud","doc":"Cloud modules start a supervisor that manages any background processes required\nby the configured adapter. Some adapters (like GCS) need authentication servers,\nwhile others (like Volume, S3) don't need any supervised processes.\n\n**Only add Cloud modules to your supervision tree if they need background processes.**\nIf you add a Cloud module that doesn't need supervision (Volume, S3), you'll see\na warning message suggesting you remove it to avoid unnecessary overhead.\n\n    # Only needed for adapters that require background processes (like GCS)\n    children = [\n      MyApp.GCSCloud  # GCS needs auth servers\n      # MyApp.VolumeCloud - Not needed, would show warning\n      # MyApp.S3Cloud - Not needed, would show warning\n    ]","ref":"Buckets.Cloud.html#module-supervision"},{"type":"behaviour","title":"Dynamic Configuration - Buckets.Cloud","doc":"For multi-tenant applications where cloud configurations are determined at runtime,\nevery Cloud module supports dynamic configuration using the process dictionary,\nsimilar to Ecto's dynamic repositories.","ref":"Buckets.Cloud.html#module-dynamic-configuration"},{"type":"behaviour","title":"Usage - Buckets.Cloud","doc":"There are two ways to use dynamic configuration:\n\n#### 1. Scoped Configuration (like Ecto transactions)\n\nUse `with_config/2` for temporary configuration:\n\n    # Define the runtime configuration\n    config = [\n      adapter: Buckets.Adapters.S3,\n      bucket: \"user-bucket\",\n      access_key_id: \"AKIA...\",\n      secret_access_key: \"secret...\",\n      region: \"us-east-1\"\n    ]\n\n    # Execute operations with the dynamic config\n    {:ok, object} = MyApp.Cloud.with_config(config, fn ->\n      MyApp.Cloud.insert(\"file.pdf\")\n      MyApp.Cloud.insert(\"another.pdf\")  # Same config\n    end)\n\n#### 2. Process-Local Configuration (like Ecto.Repo.put_dynamic_repo)\n\nUse `put_dynamic_config/1` for persistent configuration in the current process:\n\n    # Set dynamic config for this process\n    :ok = MyApp.Cloud.put_dynamic_config([\n      adapter: Buckets.Adapters.GCS,\n      bucket: \"tenant-specific-bucket\",\n      service_account_credentials: tenant.credentials\n    ])\n\n    # All subsequent operations use the dynamic config\n    {:ok, object1} = MyApp.Cloud.insert(\"file1.pdf\")\n    {:ok, object2} = MyApp.Cloud.insert(\"file2.pdf\")","ref":"Buckets.Cloud.html#module-usage"},{"type":"behaviour","title":"Auth Server Management - Buckets.Cloud","doc":"Auth servers (for GCS) are automatically started and cached per-process as needed.\nYou don't need any special configuration or supervision setup for dynamic clouds.","ref":"Buckets.Cloud.html#module-auth-server-management"},{"type":"callback","title":"Buckets.Cloud.delete/1","doc":"Deletes a `Buckets.Object` permanently.","ref":"Buckets.Cloud.html#c:delete/1"},{"type":"callback","title":"Buckets.Cloud.delete!/1","doc":"Same as `c:delete/1` but returns the object or raises if there is an error.","ref":"Buckets.Cloud.html#c:delete!/1"},{"type":"callback","title":"Buckets.Cloud.insert/2","doc":"Inserts a `Buckets.Object` or a file from a path into a bucket.","ref":"Buckets.Cloud.html#c:insert/2"},{"type":"callback","title":"Buckets.Cloud.insert!/2","doc":"Same as `c:insert/2` but returns the object or raises if there is an error.","ref":"Buckets.Cloud.html#c:insert!/2"},{"type":"callback","title":"Buckets.Cloud.live_upload/2","doc":"Returns a map to be used as configuration for a LiveView live upload. The configuration\ncontains a signed URL that permits the upload to a remote bucket. Requires that an `:uploader`\nis configured for the location that the file is being uploaded to.","ref":"Buckets.Cloud.html#c:live_upload/2"},{"type":"callback","title":"Buckets.Cloud.live_upload!/2","doc":"Same as `c:live_upload/1` but returns the upload config or raises if there is an error.","ref":"Buckets.Cloud.html#c:live_upload!/2"},{"type":"callback","title":"Buckets.Cloud.load/2","doc":"Loads the data for a `Buckets.Object`, placing it in memory by default. It will load data\nlazily, doing nothing if data is already present.","ref":"Buckets.Cloud.html#c:load/2"},{"type":"callback","title":"Options - Buckets.Cloud.load/2","doc":"* `:to` - A location to store the loaded data to on disk, if this is preferred over\n      loading to memory. May be a path, `:tmp` (to load to the configured tmp dir), or\n      `{:tmp, path}` (to load to a path in the configured tmp dir).\n    * `:force` - If data is already present, first unload it with `c:unload/1` before loading\n      new data. Warning: if data is stored in a file, it will be deleted.","ref":"Buckets.Cloud.html#c:load/2-options"},{"type":"callback","title":"Buckets.Cloud.load!/2","doc":"Same as `c:load/1` but returns the object or raises if there is an error.","ref":"Buckets.Cloud.html#c:load!/2"},{"type":"callback","title":"Buckets.Cloud.normalize_filename/1","doc":"An overridable function that processes filenames before encoding them in a remote path.\n\nBy default, replaces whitespace with `\"_\"` and removes all non-alphanumeric characters.","ref":"Buckets.Cloud.html#c:normalize_filename/1"},{"type":"callback","title":"Buckets.Cloud.read/1","doc":"Reads the data for `Buckets.Object`, preferring local data first and fetching from\nthe remote bucket if needed.","ref":"Buckets.Cloud.html#c:read/1"},{"type":"callback","title":"Buckets.Cloud.read!/1","doc":"Same as `c:read/1` but returns the data or raises if there is an error.","ref":"Buckets.Cloud.html#c:read!/1"},{"type":"callback","title":"Buckets.Cloud.tmp_dir/0","doc":"An overridable function that specifies the temporary directory in which objects are stored.\n\nDefaults to `System.tmp_dir!()`.","ref":"Buckets.Cloud.html#c:tmp_dir/0"},{"type":"callback","title":"Buckets.Cloud.unload/1","doc":"Unloads the data for a `Buckets.Object`. If the data is stored in a local file, the file will\nbe deleted.","ref":"Buckets.Cloud.html#c:unload/1"},{"type":"callback","title":"Buckets.Cloud.url/1","doc":"Returns a SignedURL struct for a `Buckets.Object`.","ref":"Buckets.Cloud.html#c:url/1"},{"type":"callback","title":"Buckets.Cloud.url!/2","doc":"Same as `c:url/1` but returns the SignedURL raises if there is an error.","ref":"Buckets.Cloud.html#c:url!/2"},{"type":"module","title":"Buckets.Cloud.Dynamic","doc":"Dynamic configuration management for Cloud modules.\n\nThis module provides functions for managing runtime configuration changes,\nallowing Cloud modules to switch between different adapters and configurations\non a per-process basis, similar to `Ecto.Repo.put_dynamic_repo/1`.","ref":"Buckets.Cloud.Dynamic.html"},{"type":"module","title":"Usage - Buckets.Cloud.Dynamic","doc":"These functions are automatically available on Cloud modules that use\n`Buckets.Cloud`. You typically don't need to call this module directly.","ref":"Buckets.Cloud.Dynamic.html#module-usage"},{"type":"module","title":"Process-Local Configuration - Buckets.Cloud.Dynamic","doc":"MyApp.Cloud.put_dynamic_config([\n      adapter: Buckets.Adapters.GCS,\n      bucket: \"tenant-bucket\",\n      service_account_credentials: credentials\n    ])","ref":"Buckets.Cloud.Dynamic.html#module-process-local-configuration"},{"type":"module","title":"Scoped Configuration - Buckets.Cloud.Dynamic","doc":"MyApp.Cloud.with_config(config, fn ->\n      MyApp.Cloud.insert(\"file.pdf\")\n    end)","ref":"Buckets.Cloud.Dynamic.html#module-scoped-configuration"},{"type":"module","title":"Auth Server Management - Buckets.Cloud.Dynamic","doc":"For adapters that need background processes (like GCS auth servers), these\nare automatically started and cached per-process as needed through the\nadapter's `prepare_dynamic_config/1` callback.","ref":"Buckets.Cloud.Dynamic.html#module-auth-server-management"},{"type":"function","title":"Buckets.Cloud.Dynamic.config/2","doc":"Gets the current configuration for the given cloud module.\n\nChecks the process dictionary first for dynamic config, falls back to static config.","ref":"Buckets.Cloud.Dynamic.html#config/2"},{"type":"function","title":"Buckets.Cloud.Dynamic.put_dynamic_config/2","doc":"Sets dynamic configuration for the current process.\n\nThis configuration will be used for all subsequent operations in this process\nuntil changed or deleted.","ref":"Buckets.Cloud.Dynamic.html#put_dynamic_config/2"},{"type":"function","title":"Buckets.Cloud.Dynamic.with_config/3","doc":"Executes a function with a specific configuration.\n\nThe configuration is only active during the execution of the function.\nPrevious configuration is restored afterwards.","ref":"Buckets.Cloud.Dynamic.html#with_config/3"},{"type":"module","title":"Buckets.Cloud.Supervisor","doc":"Supervisor that manages any processes required by cloud adapters.\n\nThis supervisor is used internally by Cloud modules to automatically\nstart any required processes for configured adapters by calling the\nadapter's `child_spec/1` callback. Adapters that don't need supervised\nprocesses (like Volume, S3) return `nil`, while adapters that need\nbackground processes (like GCS auth servers) return proper child specs.","ref":"Buckets.Cloud.Supervisor.html"},{"type":"function","title":"Buckets.Cloud.Supervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Buckets.Cloud.Supervisor.html#child_spec/1"},{"type":"function","title":"Buckets.Cloud.Supervisor.start_link/1","doc":"Starts the supervisor with a Cloud module.","ref":"Buckets.Cloud.Supervisor.html#start_link/1"},{"type":"module","title":"Buckets.Location","doc":"Represents a storage location for a Buckets.Object.\n\nThe config field can be either:\n- A keyword list with adapter configuration\n- A Cloud module atom that implements the Buckets.Cloud behaviour","ref":"Buckets.Location.html"},{"type":"function","title":"Buckets.Location.get_config/1","doc":"Gets the configuration from a Location.\n\nIf the config is a keyword list, returns it as-is.\nIf the config is a Cloud module, calls the module's config/0 function.","ref":"Buckets.Location.html#get_config/1"},{"type":"function","title":"Buckets.Location.new/2","doc":"Creates a new Location struct with the given path and configuration.","ref":"Buckets.Location.html#new/2"},{"type":"function","title":"Parameters - Buckets.Location.new/2","doc":"- `path` - The storage path for the object (e.g., \"uploads/123/file.pdf\")\n- `config` - Either:\n  - A keyword list with adapter configuration\n  - A Cloud module atom that implements the Buckets.Cloud behaviour","ref":"Buckets.Location.html#new/2-parameters"},{"type":"function","title":"Examples - Buckets.Location.new/2","doc":"# With explicit configuration\n    location = Location.new(\"uploads/file.pdf\", [\n      adapter: Buckets.Adapters.S3,\n      bucket: \"my-bucket\",\n      region: \"us-east-1\"\n    ])\n\n    # With Cloud module reference\n    location = Location.new(\"uploads/file.pdf\", MyApp.Cloud)","ref":"Buckets.Location.html#new/2-examples"},{"type":"function","title":"Returns - Buckets.Location.new/2","doc":"A `%Buckets.Location{}` struct with the path and configuration set.","ref":"Buckets.Location.html#new/2-returns"},{"type":"type","title":"Buckets.Location.t/0","doc":"","ref":"Buckets.Location.html#t:t/0"},{"type":"module","title":"Buckets.Location.NotConfigured","doc":"Represents an object that hasn't been assigned a storage location yet.\n\nThis struct is used as the default location for newly created `Buckets.Object`\ninstances that haven't been uploaded to any storage backend. It serves as a\nmarker to indicate that the object exists only in memory or as a local file,\nbut hasn't been persisted to cloud storage.","ref":"Buckets.Location.NotConfigured.html"},{"type":"module","title":"Usage - Buckets.Location.NotConfigured","doc":"Objects with a `NotConfigured` location:\n- Cannot be read from remote storage\n- Cannot generate signed URLs\n- Must be inserted into a Cloud module before remote operations","ref":"Buckets.Location.NotConfigured.html#module-usage"},{"type":"module","title":"Examples - Buckets.Location.NotConfigured","doc":"# New objects start with NotConfigured location\n    object = Buckets.Object.new(\"123\", \"document.pdf\")\n    # object.location == %Buckets.Location.NotConfigured{}\n\n    # After insertion, location is configured\n    {:ok, stored} = MyApp.Cloud.insert(object)\n    # stored.location == %Buckets.Location{path: \"...\", config: ...}","ref":"Buckets.Location.NotConfigured.html#module-examples"},{"type":"module","title":"Buckets.Object","doc":"Represents a file object in the Buckets storage system.\n\nAn Object encapsulates file data, metadata, and location information. Objects\ncan exist in various states:","ref":"Buckets.Object.html"},{"type":"module","title":"Object States - Buckets.Object","doc":"1. **Local only** - Created from a file or upload, not yet stored remotely\n   - `stored?: false`\n   - `location: %Buckets.Location.NotConfigured{}`\n   - `data: {:file, path}` or `{:data, binary}`\n\n2. **Stored remotely** - Uploaded to cloud storage\n   - `stored?: true`\n   - `location: %Buckets.Location{}`\n   - `data: nil` (unless loaded)\n\n3. **Loaded** - Remote object with data fetched\n   - `stored?: true`\n   - `location: %Buckets.Location{}`\n   - `data: {:file, path}` or `{:data, binary}`","ref":"Buckets.Object.html#module-object-states"},{"type":"module","title":"Creating Objects - Buckets.Object","doc":"Objects can be created from:\n- Files on disk: `from_file/1`\n- Plug uploads: `from_upload/1` with `%Plug.Upload{}`\n- LiveView uploads: `from_upload/1` with `%Phoenix.LiveView.UploadEntry{}`\n- Manually: `new/3`","ref":"Buckets.Object.html#module-creating-objects"},{"type":"module","title":"Metadata - Buckets.Object","doc":"Objects store metadata including:\n- `:content_type` - MIME type of the file\n- `:content_size` - Size in bytes\n- Custom metadata can be added as needed","ref":"Buckets.Object.html#module-metadata"},{"type":"module","title":"Example Usage - Buckets.Object","doc":"# Create from file\n    object = Buckets.Object.from_file(\"/path/to/document.pdf\")\n\n    # Upload to storage\n    {:ok, stored} = MyApp.Cloud.insert(object)\n\n    # Read data from stored object\n    {:ok, data} = MyApp.Cloud.read(stored)\n\n    # Load remote object data\n    {:ok, loaded} = MyApp.Cloud.load(stored)\n    {:ok, data} = Buckets.Object.read(loaded)","ref":"Buckets.Object.html#module-example-usage"},{"type":"function","title":"Buckets.Object.from_file/1","doc":"Creates an Object from a file on disk.\n\nAutomatically generates a UUID and extracts metadata including content type\n(from file extension) and content size.","ref":"Buckets.Object.html#from_file/1"},{"type":"function","title":"Parameters - Buckets.Object.from_file/1","doc":"- `path` - Path to the file on disk","ref":"Buckets.Object.html#from_file/1-parameters"},{"type":"function","title":"Raises - Buckets.Object.from_file/1","doc":"Raises if the file does not exist.","ref":"Buckets.Object.html#from_file/1-raises"},{"type":"function","title":"Examples - Buckets.Object.from_file/1","doc":"object = Buckets.Object.from_file(\"/tmp/upload.pdf\")\n    # %Buckets.Object{\n    #   uuid: \"generated-uuid\",\n    #   filename: \"upload.pdf\",\n    #   data: {:file, \"/tmp/upload.pdf\"},\n    #   metadata: %{\n    #     content_type: \"application/pdf\",\n    #     content_size: 12345\n    #   },\n    #   stored?: false\n    # }","ref":"Buckets.Object.html#from_file/1-examples"},{"type":"function","title":"Buckets.Object.from_upload/1","doc":"Creates an Object from various upload types.\n\nThis function has multiple clauses to handle different upload scenarios:","ref":"Buckets.Object.html#from_upload/1"},{"type":"function","title":"Plug.Upload - Buckets.Object.from_upload/1","doc":"Creates an Object from a `%Plug.Upload{}` struct (Phoenix controller uploads).\nAutomatically extracts metadata including content type and size.\n\n    def create(conn, %{\"file\" => upload}) do\n      object = Buckets.Object.from_upload(upload)\n      {:ok, stored} = MyApp.Cloud.insert(object)\n    end","ref":"Buckets.Object.html#from_upload/1-plug-upload"},{"type":"function","title":"LiveView UploadEntry (incomplete) - Buckets.Object.from_upload/1","doc":"Creates an Object from a `%Phoenix.LiveView.UploadEntry{}` that is not yet complete.\nUsed for generating signed URLs for direct uploads.\n\n    # In LiveView for generating upload URLs\n    object = Buckets.Object.from_upload(entry)\n    {:ok, config} = MyApp.Cloud.live_upload(entry)\n\nRaises if called with a completed upload entry without metadata.","ref":"Buckets.Object.html#from_upload/1-liveview-uploadentry-incomplete"},{"type":"function","title":"LiveView UploadEntry (completed) - Buckets.Object.from_upload/1","doc":"Creates an Object from a completed upload with metadata tuple `{entry, meta}`.\nThe metadata typically includes the uploaded file path and/or signed URL.\n\n    # After LiveView upload completes\n    consume_uploaded_entries(socket, :avatar, fn %{path: path}, entry ->\n      object = Buckets.Object.from_upload({entry, %{path: path}})\n      {:ok, object}\n    end)\n\nRaises if called with an incomplete upload entry.","ref":"Buckets.Object.html#from_upload/1-liveview-uploadentry-completed"},{"type":"function","title":"Buckets.Object.new/3","doc":"Creates a new Object with the given UUID, filename, and options.","ref":"Buckets.Object.html#new/3"},{"type":"function","title":"Parameters - Buckets.Object.new/3","doc":"- `uuid` - Unique identifier for the object\n- `filename` - Original filename\n- `opts` - Keyword list of options:\n  - `:location` - Either `{path, config}` tuple or `%Buckets.Location{}` struct\n  - `:metadata` - Map of metadata (defaults to `%{}`)","ref":"Buckets.Object.html#new/3-parameters"},{"type":"function","title":"Examples - Buckets.Object.new/3","doc":"# Basic object\n    object = Buckets.Object.new(\"123\", \"document.pdf\", [])\n\n    # With location\n    object = Buckets.Object.new(\"123\", \"document.pdf\",\n      location: {\"uploads/123/document.pdf\", MyApp.Cloud}\n    )\n\n    # With metadata\n    object = Buckets.Object.new(\"123\", \"document.pdf\",\n      metadata: %{content_type: \"application/pdf\", author: \"John\"}\n    )","ref":"Buckets.Object.html#new/3-examples"},{"type":"function","title":"Buckets.Object.read/1","doc":"Reads the data from an object.\n\nReturns `{:ok, data}` if the object has data loaded, either in memory\nor from a file. Returns `{:error, :not_loaded}` if no data is available.","ref":"Buckets.Object.html#read/1"},{"type":"function","title":"Examples - Buckets.Object.read/1","doc":"# Object with data in memory\n    {:ok, binary_data} = Buckets.Object.read(object)\n\n    # Object without loaded data\n    {:error, :not_loaded} = Buckets.Object.read(remote_object)","ref":"Buckets.Object.html#read/1-examples"},{"type":"function","title":"Note - Buckets.Object.read/1","doc":"For remote objects without loaded data, use `Cloud.load/2` first to fetch\nthe data from storage.","ref":"Buckets.Object.html#read/1-note"},{"type":"function","title":"Buckets.Object.read!/1","doc":"Reads the data from an object, raising on error.\n\nSame as `read/1` but raises an exception if the data cannot be read.","ref":"Buckets.Object.html#read!/1"},{"type":"function","title":"Raises - Buckets.Object.read!/1","doc":"- Raises if the object has no data loaded\n- Raises if the file cannot be read from disk","ref":"Buckets.Object.html#read!/1-raises"},{"type":"function","title":"Examples - Buckets.Object.read!/1","doc":"# Success case\n    binary_data = Buckets.Object.read!(object)\n\n    # Raises for unloaded object\n    Buckets.Object.read!(remote_object)\n    # ** (RuntimeError) Called `read!/1` with object that does not have data loaded.","ref":"Buckets.Object.html#read!/1-examples"},{"type":"type","title":"Buckets.Object.t/0","doc":"","ref":"Buckets.Object.html#t:t/0"},{"type":"module","title":"Buckets.Router","doc":"Router helpers for integrating Buckets with Phoenix applications.\n\nThis module provides macros for setting up development routes that handle\nfile uploads and downloads when using the Volume adapter. These routes\nenable direct file uploads to the local filesystem during development,\nmimicking the behavior of cloud storage providers.","ref":"Buckets.Router.html"},{"type":"module","title":"Usage - Buckets.Router","doc":"Import this module in your Phoenix router and use the `buckets_volume/2`\nmacro to set up the necessary routes:\n\n    # In your router.ex\n    defmodule MyAppWeb.Router do\n      use MyAppWeb, :router\n\n      # Only include in development\n      if Application.compile_env(:my_app, :dev_routes) do\n        import Buckets.Router\n\n        # Sets up routes at /__buckets__/:bucket/*path\n        buckets_volume(MyApp.Cloud)\n      end\n    end","ref":"Buckets.Router.html#module-usage"},{"type":"module","title":"Routes Created - Buckets.Router","doc":"The macro creates two routes under the `/__buckets__` scope:\n- `GET /__buckets__/:bucket/*path` - Download files\n- `PUT /__buckets__/:bucket/*path` - Upload files","ref":"Buckets.Router.html#module-routes-created"},{"type":"module","title":"Security - Buckets.Router","doc":"These routes validate:\n- The bucket name matches the configured bucket\n- Upload requests have valid signatures (generated by the Volume adapter)","ref":"Buckets.Router.html#module-security"},{"type":"module","title":"Configuration - Buckets.Router","doc":"The routes can be customized with options:\n\n    buckets_volume(MyApp.Cloud, cache_control: 3600)\n\nAvailable options are passed to the controller and may include:\n- `:cache_control` - Max age in seconds for cache control headers","ref":"Buckets.Router.html#module-configuration"},{"type":"macro","title":"Buckets.Router.buckets_volume/2","doc":"Sets up Volume adapter routes for local file handling in development.\n\nThis macro creates GET and PUT routes under the `/__buckets__` scope that\nhandle file downloads and uploads respectively. It's designed to be used\nwith the Volume adapter during development to provide a local alternative\nto cloud storage.","ref":"Buckets.Router.html#buckets_volume/2"},{"type":"macro","title":"Parameters - Buckets.Router.buckets_volume/2","doc":"- `cloud_module` - The Cloud module to use for configuration (e.g., `MyApp.Cloud`)\n- `opts` - Optional keyword list of options:\n  - `:cache_control` - Max age in seconds for cache control headers","ref":"Buckets.Router.html#buckets_volume/2-parameters"},{"type":"macro","title":"Examples - Buckets.Router.buckets_volume/2","doc":"# Basic usage\n    buckets_volume(MyApp.Cloud)\n\n    # With cache control\n    buckets_volume(MyApp.Cloud, cache_control: 3600)","ref":"Buckets.Router.html#buckets_volume/2-examples"},{"type":"macro","title":"Routes Created - Buckets.Router.buckets_volume/2","doc":"The following routes are created:\n- `GET /__buckets__/:bucket/*path` - Downloads a file from the local filesystem\n- `PUT /__buckets__/:bucket/*path` - Uploads a file to the local filesystem\n\nBoth routes include the cloud module and options in the connection's private\ndata for use by the VolumeController.","ref":"Buckets.Router.html#buckets_volume/2-routes-created"},{"type":"macro","title":"Security Note - Buckets.Router.buckets_volume/2","doc":"These routes should only be used in development. The macro includes validation\nto ensure the bucket parameter matches the configured bucket and that uploads\ninclude valid signatures.","ref":"Buckets.Router.html#buckets_volume/2-security-note"},{"type":"function","title":"Buckets.Router.scope/0","doc":"Returns the default scope path used for Buckets routes.\n\nThis is useful if you need to reference the scope in your own code or\nneed to know where the routes are mounted.","ref":"Buckets.Router.html#scope/0"},{"type":"function","title":"Examples - Buckets.Router.scope/0","doc":"iex> Buckets.Router.scope()\n    \"__buckets__\"","ref":"Buckets.Router.html#scope/0-examples"},{"type":"module","title":"Buckets.Router.VolumeController","doc":"Phoenix controller for handling local file uploads and downloads in development.\n\nThis controller is automatically used by the `Buckets.Router.buckets_volume/2`\nmacro to handle Volume adapter file operations.","ref":"Buckets.Router.VolumeController.html"},{"type":"module","title":"Endpoints - Buckets.Router.VolumeController","doc":"- `get/2` - Downloads files from the local filesystem\n- `put/2` - Uploads files to the local filesystem with signature validation\n\nFor usage and configuration, see `Buckets.Router`.","ref":"Buckets.Router.VolumeController.html#module-endpoints"},{"type":"function","title":"Buckets.Router.VolumeController.get/2","doc":"","ref":"Buckets.Router.VolumeController.html#get/2"},{"type":"function","title":"Buckets.Router.VolumeController.put/2","doc":"","ref":"Buckets.Router.VolumeController.html#put/2"},{"type":"module","title":"Buckets.SignedURL","doc":"Represents a signed URL for direct access to objects in cloud storage.\n\nSigned URLs provide temporary, secure access to objects without requiring\nauthentication. They are commonly used for:\n- Direct file downloads from cloud storage\n- Direct uploads to cloud storage (especially with LiveView)\n- Temporary sharing of private files","ref":"Buckets.SignedURL.html"},{"type":"module","title":"Structure - Buckets.SignedURL","doc":"A SignedURL contains:\n- `:url` - The complete signed URL string that can be used for access\n- `:location` - The `Buckets.Location` struct indicating where the object is stored","ref":"Buckets.SignedURL.html#module-structure"},{"type":"module","title":"Usage - Buckets.SignedURL","doc":"SignedURLs are typically generated by adapters through the `url/2` callback:\n\n    {:ok, signed_url} = MyApp.Cloud.url(object, expires_in: 3600)\n\n    # Access the URL string\n    url_string = signed_url.url\n    # or use String.Chars protocol\n    url_string = to_string(signed_url)","ref":"Buckets.SignedURL.html#module-usage"},{"type":"module","title":"LiveView Uploads - Buckets.SignedURL","doc":"SignedURLs are used for direct uploads in LiveView:\n\n    {:ok, config} = MyApp.Cloud.live_upload(entry)\n    # config.url is a SignedURL for uploading","ref":"Buckets.SignedURL.html#module-liveview-uploads"},{"type":"module","title":"JSON Encoding - Buckets.SignedURL","doc":"SignedURLs implement the `Jason.Encoder` protocol and encode to just the URL string:\n\n    Jason.encode!(%{upload_url: signed_url})\n    # {\"upload_url\": \"https://storage.example.com/bucket/file?signature=...\"}","ref":"Buckets.SignedURL.html#module-json-encoding"},{"type":"module","title":"Security Considerations - Buckets.SignedURL","doc":"- Signed URLs typically expire after a set time period\n- They should be treated as sensitive data\n- Different adapters have different expiration limits and security features","ref":"Buckets.SignedURL.html#module-security-considerations"},{"type":"function","title":"Buckets.SignedURL.to_string/1","doc":"Converts a SignedURL to its URL string representation.\n\nThis function extracts just the URL string from the SignedURL struct.\nIt's also used by the `String.Chars` protocol implementation.","ref":"Buckets.SignedURL.html#to_string/1"},{"type":"function","title":"Examples - Buckets.SignedURL.to_string/1","doc":"iex> signed_url = %Buckets.SignedURL{url: \"https://example.com/file?sig=123\"}\n    iex> Buckets.SignedURL.to_string(signed_url)\n    \"https://example.com/file?sig=123\"\n\n    # Also works with Kernel.to_string/1\n    iex> to_string(signed_url)\n    \"https://example.com/file?sig=123\"","ref":"Buckets.SignedURL.html#to_string/1-examples"},{"type":"type","title":"Buckets.SignedURL.t/0","doc":"","ref":"Buckets.SignedURL.html#t:t/0"},{"type":"module","title":"Buckets.Telemetry","doc":"Telemetry events for Buckets.\n\nThis module defines telemetry events that other applications can attach to.","ref":"Buckets.Telemetry.html"},{"type":"module","title":"Event Format - Buckets.Telemetry","doc":"Events follow the convention: `[:buckets, component, operation, stage]`","ref":"Buckets.Telemetry.html#module-event-format"},{"type":"module","title":"Events - Buckets.Telemetry","doc":"","ref":"Buckets.Telemetry.html#module-events"},{"type":"module","title":"Cloud Operations - Buckets.Telemetry","doc":"* `[:buckets, :cloud, :insert, :start]` - Emitted when a file upload operation starts\n* `[:buckets, :cloud, :insert, :stop]` - Emitted when a file upload operation completes\n* `[:buckets, :cloud, :insert, :exception]` - Emitted when a file upload operation raises an exception\n* `[:buckets, :cloud, :delete, :start]` - Emitted when a file deletion operation starts\n* `[:buckets, :cloud, :delete, :stop]` - Emitted when a file deletion operation completes\n* `[:buckets, :cloud, :delete, :exception]` - Emitted when a file deletion operation raises an exception\n* `[:buckets, :cloud, :read, :start]` - Emitted when a file read operation starts\n* `[:buckets, :cloud, :read, :stop]` - Emitted when a file read operation completes\n* `[:buckets, :cloud, :read, :exception]` - Emitted when a file read operation raises an exception\n* `[:buckets, :cloud, :load, :start]` - Emitted when a file load operation starts\n* `[:buckets, :cloud, :load, :stop]` - Emitted when a file load operation completes\n* `[:buckets, :cloud, :load, :exception]` - Emitted when a file load operation raises an exception\n* `[:buckets, :cloud, :url, :start]` - Emitted when a URL generation operation starts\n* `[:buckets, :cloud, :url, :stop]` - Emitted when a URL generation operation completes\n* `[:buckets, :cloud, :url, :exception]` - Emitted when a URL generation operation raises an exception","ref":"Buckets.Telemetry.html#module-cloud-operations"},{"type":"module","title":"Adapter Operations - Buckets.Telemetry","doc":"* `[:buckets, :adapter, :put, :start]` - Emitted when an adapter put operation starts\n* `[:buckets, :adapter, :put, :stop]` - Emitted when an adapter put operation completes\n* `[:buckets, :adapter, :put, :exception]` - Emitted when an adapter put operation raises an exception\n* `[:buckets, :adapter, :get, :start]` - Emitted when an adapter get operation starts\n* `[:buckets, :adapter, :get, :stop]` - Emitted when an adapter get operation completes\n* `[:buckets, :adapter, :get, :exception]` - Emitted when an adapter get operation raises an exception\n* `[:buckets, :adapter, :url, :start]` - Emitted when an adapter URL generation starts\n* `[:buckets, :adapter, :url, :stop]` - Emitted when an adapter URL generation completes\n* `[:buckets, :adapter, :url, :exception]` - Emitted when an adapter URL generation raises an exception\n* `[:buckets, :adapter, :delete, :start]` - Emitted when an adapter delete operation starts\n* `[:buckets, :adapter, :delete, :stop]` - Emitted when an adapter delete operation completes\n* `[:buckets, :adapter, :delete, :exception]` - Emitted when an adapter delete operation raises an exception","ref":"Buckets.Telemetry.html#module-adapter-operations"},{"type":"module","title":"Authentication Operations - Buckets.Telemetry","doc":"* `[:buckets, :auth, :token, :fetch, :start]` - Emitted when token fetching starts\n* `[:buckets, :auth, :token, :fetch, :stop]` - Emitted when token fetching completes\n* `[:buckets, :auth, :token, :fetch, :exception]` - Emitted when token fetching raises an exception\n* `[:buckets, :auth, :token, :refresh, :start]` - Emitted when token refresh starts\n* `[:buckets, :auth, :token, :refresh, :stop]` - Emitted when token refresh completes\n* `[:buckets, :auth, :token, :refresh, :exception]` - Emitted when token refresh raises an exception","ref":"Buckets.Telemetry.html#module-authentication-operations"},{"type":"module","title":"Usage - Buckets.Telemetry","doc":"To attach to these events in your application:\n\n```elixir\n:telemetry.attach(\n  \"my-handler-id\",\n  [:buckets, :cloud, :insert, :start],\n  &MyApp.handle_cloud_insert/4,\n  nil\n)\n\ndef handle_cloud_insert(_event_name, measurements, metadata, _config) do\n  # Process the event\n  IO.inspect(measurements)\n  IO.inspect(metadata)\nend\n```\n\nYou may attach a default logger handler, if you want basic logs for all telemetry events:\n\n```elixir\nBuckets.Telemetry.attach_default_logger(:debug)\n```","ref":"Buckets.Telemetry.html#module-usage"},{"type":"function","title":"Buckets.Telemetry.attach_default_logger/1","doc":"Attaches a default structured JSON Telemetry handler for logging.","ref":"Buckets.Telemetry.html#attach_default_logger/1"},{"type":"function","title":"Buckets.Telemetry.default_handler_id/0","doc":"","ref":"Buckets.Telemetry.html#default_handler_id/0"},{"type":"function","title":"Buckets.Telemetry.detach_default_logger/0","doc":"Undoes `Buckets.Telemetry.attach_default_logger/1` by detaching the attached logger.","ref":"Buckets.Telemetry.html#detach_default_logger/0"},{"type":"function","title":"Examples - Buckets.Telemetry.detach_default_logger/0","doc":"Detach a previously attached logger:\n\n    :ok = Buckets.Telemetry.attach_default_logger()\n    :ok = Buckets.Telemetry.detach_default_logger()\n\nAttempt to detach when a logger wasn't attached:\n\n    {:error, :not_found} = Buckets.Telemetry.detach_default_logger()","ref":"Buckets.Telemetry.html#detach_default_logger/0-examples"},{"type":"function","title":"Buckets.Telemetry.emit_event/3","doc":"Emits a telemetry event with the given name, measurements, and metadata.","ref":"Buckets.Telemetry.html#emit_event/3"},{"type":"function","title":"Buckets.Telemetry.handle_event/4","doc":"Default telemetry event handler that logs events.","ref":"Buckets.Telemetry.html#handle_event/4"},{"type":"function","title":"Buckets.Telemetry.span/3","doc":"Wraps a function call with start and stop telemetry events.","ref":"Buckets.Telemetry.html#span/3"},{"type":"function","title":"Buckets.Telemetry.start_event/2","doc":"Emits a start event and returns a function to emit the corresponding stop event.\n\nThis is useful for span-like events where you want to measure the duration of an operation.","ref":"Buckets.Telemetry.html#start_event/2"},{"type":"module","title":"Buckets.Util","doc":"Utility functions for the Buckets library.\n\nThis module provides helper functions used internally by Buckets for\nfile operations and string manipulation.","ref":"Buckets.Util.html"},{"type":"function","title":"Buckets.Util.normalize_filename/1","doc":"Normalizes a filename for safe storage in cloud systems.\n\nThis function sanitizes filenames by:\n- Replacing whitespace with underscores\n- Removing all characters except letters, numbers, dots, underscores, and hyphens\n\nThis normalization helps prevent issues with special characters in different\nstorage systems and URLs.\n\nThis function is used by Cloud modules when generating storage paths. It can be overridden\nin your Cloud module if you need different normalization rules:\n\n    defmodule MyApp.Cloud do\n      use Buckets.Cloud, otp_app: :my_app\n\n      def normalize_filename(filename) do\n        # Custom normalization logic\n      end\n    end","ref":"Buckets.Util.html#normalize_filename/1"},{"type":"function","title":"Examples - Buckets.Util.normalize_filename/1","doc":"iex> Buckets.Util.normalize_filename(\"My Document.pdf\")\n    \"My_Document.pdf\"\n\n    iex> Buckets.Util.normalize_filename(\"invoice #123 (final).xlsx\")\n    \"invoice_123_final.xlsx\"\n\n    iex> Buckets.Util.normalize_filename(\"photo@event!.jpg\")\n    \"photoevent.jpg\"","ref":"Buckets.Util.html#normalize_filename/1-examples"},{"type":"function","title":"Buckets.Util.size/1","doc":"Gets the size of a file in bytes. Raises if the file does not exist or cannot be accessed","ref":"Buckets.Util.html#size/1"},{"type":"function","title":"Examples - Buckets.Util.size/1","doc":"iex> Buckets.Util.size(\"/path/to/file.pdf\")\n    12345\n\n    iex> Buckets.Util.size(\"non_existent.txt\")\n    ** (File.Error) could not read file stats \"non_existent.txt\": no such file or directory","ref":"Buckets.Util.html#size/1-examples"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,49]}}